SystemOrganization addCategory: #'GT-Inspector'!

!Class methodsFor: '*GT-Inspector' stamp: 'TudorGirba 1/28/2014 23:10'!
gtInspectorExamplesIn: composite
	"This provides a list of example objects for the current class"
	<gtInspectorPresentationOrder: 20>
	composite custom: (GTExamplesBrowser new startOn: self)! !

GLMCompositePresentation subclass: #GTExamplesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTExamplesBrowser commentStamp: 'TudorGirba 3/17/2014 07:53' prior: 0!
A simple browser showing the list of examples for a given class.

For example:
self openOn: FileReference!

!GTExamplesBrowser methodsFor: 'building' stamp: 'TudorGirba 3/17/2014 07:54'!
compose
	self title: 'E.g.'.
	self when: [ :class | (self examplesFor: class) notEmpty ].
	self list
		display: [ :class | self examplesFor: class ];
		format: [ :pragmaExamplePair | pragmaExamplePair result gtDisplayString ];
		send: [ :pragmaExamplePair | pragmaExamplePair result ];
		morphicSelectionAct: [ :list | 
			list rawSelection methodReference browse ] 
			icon: GLMUIThemeExtraIcons glamorousBrowse 
			on: $b 
			entitled: 'Browse source';
		selectionPopulate: #selection 
			on: $o 
			entitled: 'Open source' 
			with: [ :list | list rawSelection methodReference ]! !

!GTExamplesBrowser methodsFor: 'private' stamp: 'TudorGirba 3/17/2014 07:52'!
examplesFor: anObject
	| pragmas |
	pragmas := Pragma 
				allNamed: #gtExample 
				in: anObject class.
	^ pragmas collect: [ :eachPragma | 
		GTExample new
			methodReference: eachPragma method asRingDefinition ]
! !

GLMCompositePresentation subclass: #GTInspector
	instanceVariableNames: ''
	classVariableNames: 'GTNumberOfInspectorPanes'
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspector commentStamp: 'TudorGirba 8/21/2011 16:57' prior: 0!
This is meant to be used for inspecting objects. It offers multiple views and it uses a finder as a navigation. One particular feature is that you can use the evaluator tab to enter code, and evaluating it results in opening another pane to the right.

The object finder asks dynamically the object for the actual presentations that are displayed in each pane.

Example:
self openOn: Smalltalk.

Register it as a replacement for explorer:
self registerToolsOn: Smalltalk tools.!

!GTInspector class methodsFor: 'openning' stamp: 'TudorGirba 11/1/2013 22:16'!
explore: anObject 
	self openOn: anObject.
	^ anObject! !

!GTInspector class methodsFor: 'inspecting' stamp: 'TudorGirba 3/8/2014 07:51'!
gtInspectorExtensionsIn: composite
	<gtInspectorPresentationOrder: 40>
	composite list
		title: 'Extensions';
		display: [ self inspectorExtendingMethods ];
		showOnly: 50;
		format: [ :each | each gtDisplayString ];
		filterOn: [ :text :each | each gtDisplayString includesSubstring: text caseSensitive: false ]! !

!GTInspector class methodsFor: 'class initialization' stamp: 'TudorGirba 10/7/2012 22:50'!
initialize 
	self numberOfInspectorPanes: 2! !

!GTInspector class methodsFor: 'openning' stamp: 'TudorGirba 11/1/2013 22:16'!
inspect: anObject 
	self openOn: anObject.
	^ anObject! !

!GTInspector class methodsFor: 'openning' stamp: 'AndreiChis 4/16/2014 18:15'!
inspect: anObject label: aString
	self new 
		title: aString;
		openOn: anObject.
	^ anObject
! !

!GTInspector class methodsFor: 'openning' stamp: 'AndreHora 11/2/2013 00:17'!
inspector: anObject

	^ self inspect: anObject! !

!GTInspector class methodsFor: 'inspecting' stamp: 'TudorGirba 2/7/2014 08:31'!
inspectorExtendingMethods
	^ (Object withAllSubclasses
		flatCollect: [ :each | 
			(Pragma allNamed: #gtInspectorPresentationOrder: in: each) 
				collect: [ :p | p method asRingDefinition ] ])
		sorted! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 10/7/2012 22:46'!
numberOfInspectorPanes
	^ GTNumberOfInspectorPanes ! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 10/7/2012 22:46'!
numberOfInspectorPanes: anInteger
	GTNumberOfInspectorPanes := anInteger ! !

!GTInspector class methodsFor: 'openning' stamp: 'TudorGirba 11/25/2012 22:53'!
openOn: anObject 
	^ self new openOn: anObject! !

!GTInspector class methodsFor: 'tools registry' stamp: 'TudorGirba 11/1/2013 22:17'!
registerToolsOn: registry
	"self registerToolsOn: Smalltalk tools " 
	registry register: self as: #inspector
! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 10/8/2012 20:13'!
settingsOn: aBuilder 
	<systemsettings>
	
	(aBuilder group: #glamorousInspector)
		label: 'Glamorous Inspector';
		with: [
			(aBuilder setting: #numberOfInspectorPanes)
				target: self; 
				label: 'Number of inspector panes';
		 		description: 'Specifies the maximum number of panes that are visible at every moment in a Glamorous Inspector.'.]! !

!GTInspector methodsFor: 'building' stamp: 'TudorGirba 4/24/2014 22:52'!
compose
	self title: [:anObject | 'Inspector on ', (self printObjectAsAnItem: anObject)].
	self act: [:b | b update] icon: GLMUIThemeExtraIcons glamorousRefresh entitled: 'Update'.
	self pager with: [ :browser | 
		browser fixedSizePanes: self class numberOfInspectorPanes.
		browser show: [ :a :each |
			a title: [ self printObjectAsAnItem: each ].
			a dynamicActions: [ each gtInspectorActions ].
			a dynamic display: [ :x | 
				GLMCompositePresentation new with: [:d | 
					each 
						gtInspectorPresentationsIn: d 
						inContext: self ] ] ] ]! !

!GTInspector methodsFor: 'accessing' stamp: 'TudorGirba 3/13/2014 13:39'!
entities
	"returns the list of objects from the #entity port of each pane of the finder"
	^ self first panes collect: [ :each | 
		(each port: #entity) value ]! !

!GTInspector methodsFor: 'opening' stamp: 'TudorGirba 8/21/2011 08:56'!
openExplorerFor: anObject
	"this method is needed for compatibility with ObjectExplorer"
	self openOn: anObject! !

!GTInspector methodsFor: 'accessing' stamp: 'TudorGirba 3/17/2014 16:30'!
panes
	^ self first panes! !

!GTInspector methodsFor: 'private' stamp: 'TudorGirba 1/28/2014 22:59'!
printObjectAsAnItem: anObject 
	|  stream  |
	stream := String new writeStream.
	stream
		nextPutAll: (anObject class name first isVowel 
				ifTrue: [ 'an ' ]
				ifFalse: [ 'a ' ]);
		nextPutAll: anObject class name.
	stream 
		nextPutAll: ' (';
		nextPutAll: (GTObjectPrinter new asTruncatedTextFrom: anObject) asString;
		nextPut: $).
	^ stream contents! !

GLMCompositePresentation subclass: #GTObjectVariablesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTObjectVariablesBrowser commentStamp: 'TudorGirba 8/22/2011 12:52' prior: 0!
This is a helper browser for the main view in the GTInspector.

Example:
self openOn: Smalltalk!

!GTObjectVariablesBrowser methodsFor: 'building' stamp: 'TudorGirba 12/21/2013 22:38'!
compose
	self title: 'State'.
	self tabulator with: [ :browser |
		browser row: #variables; row: #evaluator.
		browser transmit to: #variables; andShow: [:a | 
			self variablesIn: a ].
		browser transmit to: #evaluator; andShow: [:a | 
			self evaluatorIn: a ].
		browser transmit from: #variables; toOutsidePort: #selection.
		browser transmit from: #evaluator; toOutsidePort: #selection ]! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 5/21/2014 10:52'!
evaluatorIn: composite
	^ (composite smalltalkCode)
		beForScripting;
		doItReceiver: #yourself;
		display: [ :object | 
			String streamContents: [ :stream | 
				stream 
					nextPutAll: (self printStringOf: object);
					cr;
					nextPutAll: (GTSnippets snippetAt: object class)] ];
		selectionAct: [ :text :object |
				GTSnippets snippetAt: object class put: (self stringWithoutInitialCommentFrom: text text asString).
				text evaluateSelectionAndDo: [ :result | text selection: result ] ]
			on: $o
			entitled: 'Do it and open (o)';
		installDefaultSelectionActions! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'TudorGirba 11/27/2013 14:53'!
printStringOf: anObject
	^ [(anObject printString contractTo: 200) asSmalltalkComment] 
			on: Error 
			do: ['"Error printing. Try self printString to debug"']! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'TudorGirba 2/2/2014 15:01'!
stringWithoutInitialCommentFrom: aString
	^ aString first = $" 
		ifFalse: [ aString trimBoth ]
		ifTrue: [ 
			(aString
				copyFrom: (aString findAnySubStr: '"' startingAt: 2) + 1 
				to: aString size) trimBoth ]! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'TudorGirba 12/25/2013 15:37'!
variableValuePairsFor: anObject
	^ anObject gtInspectorVariableValuePairs asSortedCollection asOrderedCollection 
		addFirst: 'self' -> anObject; 
		yourself! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'TudorGirba 5/18/2014 15:55'!
variablesIn: composite
	^ composite table
		showOnly: 50;
		display: [ :anObject | self variableValuePairsFor: anObject ];
		column: 'Variable' 
			evaluated: [:assoc | GTObjectPrinter new asNonTruncatedTextFrom: assoc key ] 
			sortedBy: [ :x :y | x > y ];
		column: 'Value' 
			evaluated: [:assoc | GTObjectPrinter new asTruncatedTextFrom: assoc value ]
			sortedBy: [ :x :y | x > y ];
		send: #value;
		morphicSelectionAct: [:list | list selection value browse ] 
			icon: GLMUIThemeExtraIcons glamorousBrowse 
			on: $b 
			entitled: 'Browse';
		morphicSelectionAct: [:list | list selection value inspect ] 
			icon: GLMUIThemeExtraIcons glamorousInspect 
			on: $i 
			entitled: 'Inspect';
		selectionAct: [:list | 
			((list selection pointersToExcept: { list selection })
									reject: [ :each | each pointsOnlyWeaklyTo: list selection ]) inspect ] 
			on: $t
			entitled: 'Open pointers to'! !

GLMCompositePresentation subclass: #GTSimpleMethodsBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTSimpleMethodsBrowser commentStamp: 'TudorGirba 1/18/2014 14:29' prior: 0!
This browser is used to browse the class structure of every object in the inspector.

Example:
self openOn: World!

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 11/24/2013 08:37'!
accumulatedSourceOf: m for: object in: composite
	composite accumulator with: [ :acc | 
		acc show: [ :a |
			a title: [ :method | method selector ].
			self sourceOf: m for: object in: a ] ]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 12/21/2013 23:05'!
classesIn: a
	^ a list
		display: [ :object | 
			| all |
			all := OrderedCollection new.
			object class withAllSuperclassesDo: [:each | all add: each ].
			all ];
"		initialize: [ :list | list selection: list entity class ];"
		format: [ :class | class name ];
		morphicSelectionAct: [ :list | Smalltalk tools browser fullOnClass: list selection ]
			icon: GLMUIThemeExtraIcons glamorousBrowse
			on: $b
			entitled: 'Browse'! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 3/9/2014 08:59'!
compose
	self title: 'Meta'.
	self tabulator with: [ :browser |
		browser
			row: [ :r | r column: #classes; column: #methods span: 3 ];
			row: #source.
		browser transmit
			to: #classes;
			andShow: [ :a | self classesIn: a ].
		browser transmit
			to: #classes port: #selection;
			transformed: [:each | each class ].
		browser transmit
			fromOutsidePort: #entity;
			from: #classes;
			to: #methods;
			andShow: [ :a | self methodsIn: a ].
		browser transmit
			from: #methods;
			fromOutsidePort: #entity;
			to: #source;
			when: [ :method | method notNil ];
			andShow: [ :a :method :object | self sourceOf: method for: object in: a ].
		browser transmit
			toOutsidePort: #selection;
			from: #methods port: #evaluatedObject.
		browser transmit
			toOutsidePort: #selection;
			from: #source port: #evaluatedObject.
	]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 7/15/2012 18:07'!
evaluationAction
	^ GLMGenericAction new
		action: [ :listPresentation :obj | 
					| result |
					result := obj perform: listPresentation selection selector.
					(listPresentation pane port: #evaluatedObject) value: result ];
		shortcut: $o;
		title: 'Do it and open';
		yourself! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 7/15/2012 18:58'!
methodActionsFor: aMethod
	^ aMethod selector isUnary
		ifTrue: [ Array with: self evaluationAction ]
		ifFalse: [ #() ]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 12/17/2013 08:00'!
methodsIn: a
	^ a list
		display: [ :object :class | class methods collect: #asRingDefinition];
		filterOn: [ :text :each | (text , '*') match: each selector ];
		tags: [ :method | method category ];
		sorted: [ :method1 :method2 | method1 selector < method2 selector ];
		format: [ :method | method selector ];
		morphicSelectionAct: [ :list :object :class | 
			Smalltalk tools browser 
				fullOnClass: class 
				selector: list selection selector ]
			icon: GLMUIThemeExtraIcons glamorousBrowse
			on: $b
			entitled: 'Browse';
		dynamicActionsOnSelection: [ :list | self methodActionsFor: list selection ]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 1/18/2014 14:24'!
sourceOf: method for: object in: composite
	^ composite smalltalkCode
		display: [ :m | m sourceCode ];
		smalltalkClass: [ :m | m methodClass ];
		doItReceiver: [ object ];
		selectionPopulate: #evaluatedObject
			on: $o
			entitled: 'Do it and open (o)'
			with: [ :text | Compiler evaluate: text selectedText for: object logged: false ];
		act: [ :text :m | self updateOuterPaneFrom: text for: method ]
			icon: GLMUIThemeExtraIcons glamorousAccept
			on: $s
			entitled: 'Accept';
		act: [ :text :m | 
			Smalltalk tools browser 
				fullOnClass: m methodClass 
				selector: m selector ]
			icon: GLMUIThemeExtraIcons glamorousBrowse
			entitled: 'Browse';
		act: [ :text :m | 
			(text pane port: #evaluatedObject) value: m ]
			icon: GLMUIThemeExtraIcons glamorousInspect
			entitled: 'Inspect'! !

!GTSimpleMethodsBrowser methodsFor: 'private' stamp: 'TudorGirba 12/17/2013 15:27'!
updateOuterPaneFrom: text for: method
	"this is a utility method to tell the pane surrounding the browser to update.
	The reason we do that is to support the use case of updating the GTInspector from
	the editor by modifying code directly in the Inspector."
	
	| outerPane newMethod |
	newMethod := method methodClass compile: text text notifying: nil.
	outerPane := text pane browser pane.
	outerPane update.
	self
		flag:
			'This is a huge hack because of so many reasons:
				- the presentation accesses the state of the outer browser directly
				- the presentation depends on the implementation of the browser 
				It should be solved through transmissions.
				
				But, it works for now and the browser is reasonably updated'.
	^ outerPane
		withAllPanesAndPresentationsDo: [ :child | 
			(child isKindOf: self class)
				ifTrue: [ 
					((child first paneNamed: #classes) port: #selection) value: method methodClass.
					((child first paneNamed: #methods) port: #selection)
						value:
							((child first paneNamed: #methods) presentations first displayedValue detect: [ :each | each selector = newMethod ]) ] ]! !

GLMCompositePresentation subclass: #GTSimpleRBTreeBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTSimpleRBTreeBrowser commentStamp: '<historical>' prior: 0!
This is a simple browser that displays the rb tree and previews each node on the source code.

Example:
self openOn: (self>>#compose) parseTree!

!GTSimpleRBTreeBrowser methodsFor: 'building' stamp: 'TudorGirba 12/21/2013 23:09'!
compose
	self title: 'Tree'.
	self tabulator with: [ :browser |
		browser
			row: #tree;
			row: #source.
		browser transmit
			to: #tree;
			andShow: [ :a | self treeIn: a ].
		browser transmit
			to: #source;
			andShow: [ :a | self sourceIn: a ].
		browser transmit
			from: #tree;
			to: #source port: #selectionInterval;
			transformed: [ :rbNode | rbNode sourceInterval ].
		browser transmit
			toOutsidePort: #selection;
			from: #tree port: #selectedNode ]! !

!GTSimpleRBTreeBrowser methodsFor: 'building' stamp: 'TudorGirba 11/29/2013 14:56'!
sourceIn: a
	^ a smalltalkCode 
		smalltalkClass: [ :each | each parents first methodClass ]; 
		display: [ :each | each source ]! !

!GTSimpleRBTreeBrowser methodsFor: 'building' stamp: 'TudorGirba 8/23/2011 01:17'!
treeIn: a
	^ a tree
		rootsExpanded;
		display: [ :anObject | {anObject} ];
		children: [ :each | each children ];
		selectionPopulate: #selectedNode
			on: $o
			entitled: 'Open'
			with: [ :tree | tree selection ];
		format: [ :each | 
					String
						streamContents: [ :stream | 
							stream
								nextPutAll: each class name;
								nextPut: $(;
								nextPutAll: ((each formattedCode truncateTo: 50) copyReplaceAll: String cr with: String space);
								nextPut: $) ] ]! !

Object subclass: #GTExample
	instanceVariableNames: 'methodReference result name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTExample commentStamp: 'TudorGirba 3/17/2014 07:51' prior: 0!
This encapsulates one example.!

!GTExample methodsFor: 'accessing' stamp: 'TudorGirba 3/17/2014 07:45'!
methodReference
	^ methodReference! !

!GTExample methodsFor: 'accessing' stamp: 'TudorGirba 3/17/2014 07:45'!
methodReference: anObject
	methodReference := anObject! !

!GTExample methodsFor: 'accessing' stamp: 'TudorGirba 3/17/2014 07:51'!
name
	^ name! !

!GTExample methodsFor: 'accessing' stamp: 'TudorGirba 3/17/2014 07:50'!
result
	^ result ifNil: [ result := self methodReference realClass theNonMetaClass perform: self methodReference selector ]! !

Object subclass: #GTObjectPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTObjectPrinter commentStamp: 'TudorGirba 1/18/2014 14:29' prior: 0!
This is a utility class for printing objects to Text or String.!

!GTObjectPrinter methodsFor: 'printing' stamp: 'TudorGirba 12/23/2013 09:17'!
asNonTruncatedTextFrom: anObject
	^ [ Text fromString: (anObject asString replaceAll: String cr with: String space) ] 
			on: Error 
			do: [ Text string: 'error printing' attribute: TextColor red ]! !

!GTObjectPrinter methodsFor: 'printing' stamp: 'TudorGirba 12/22/2013 08:00'!
asTruncatedTextFrom: anObject
	^ [ Text fromString: ((anObject gtDisplayString contractTo: 50) replaceAll: String cr with: String space) ]
		on: Error
		do: [ Text string: 'error printing' attribute: TextColor red ]! !

Object subclass: #GTSnippets
	instanceVariableNames: 'instance snippets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

GTSnippets class
	instanceVariableNames: 'instance'!

!GTSnippets commentStamp: '<historical>' prior: 0!
GTSnippets instance!

GTSnippets class
	instanceVariableNames: 'instance'!

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 14:55'!
instance
	^ instance ifNil: [ instance := self new ]! !

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 2/2/2014 14:05'!
reset
	instance := nil! !

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 15:00'!
snippetAt: anObject
	^ self instance snippetAt: anObject! !

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 15:10'!
snippetAt: anObject put: aString
	^ self instance snippetAt: anObject put: aString! !

!GTSnippets methodsFor: 'accessing' stamp: 'TudorGirba 12/30/2013 23:44'!
snippetAt: anObject
	^ self snippets 
		at: anObject 
		ifAbsent: [ 'self' ]! !

!GTSnippets methodsFor: 'accessing' stamp: 'TudorGirba 12/30/2013 23:44'!
snippetAt: anObject put: aString
	^ self snippets 
		at: anObject 
		put: aString! !

!GTSnippets methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 14:59'!
snippets 
	^ snippets ifNil: [ snippets := IdentityDictionary new ]! !

!Object methodsFor: '*gt-inspector' stamp: 'TudorGirba 12/23/2013 09:12'!
gtDisplayOn: stream
	"This offers a means to customize how the object is shown in the inspector"
	^ self printOn: stream! !

!Object methodsFor: '*gt-inspector' stamp: 'TudorGirba 12/28/2013 21:59'!
gtDisplayString
	"This offers a means to customize how the object is shown in the inspector.
	Do not override this method. Override gtDisplayOn: instead"
		
	| limitedString limit |
	limit := 1000.
	limitedString := String streamContents: [:s | self gtDisplayOn: s] limitedTo: limit.
	limitedString size < limit ifTrue: [^ limitedString].
	^ limitedString , ' ...'! !

!Object methodsFor: '*gt-inspector' stamp: 'TudorGirba 2/27/2014 21:42'!
gtInspectorActionBrowse
	<gtInspectorAction>
	^ GLMGenericAction new
		action: [ self browse ];
		icon: GLMUIThemeExtraIcons glamorousBrowse;
		title: 'Browse'! !

!Object methodsFor: '*gt-inspector' stamp: 'TudorGirba 2/27/2014 10:36'!
gtInspectorActions
	"This is a utility method that allows each object to define a set of actions 
	specific for itself. These actions are used when the object appears in the inspector.

	The actions are collected based on the #gtInspectorAction pragma that are defined in
	unary methods corresponding to the body of the actions"
	
	| all |
	all := (Pragma 
				allNamed: #gtInspectorAction
				from: self class 
				to: Object) collect: [:eachPragma |
					self perform: eachPragma selector ].
	^ all asSortedCollection: [:a :b | 
		| first second |
		first := a isCategorized ifTrue: [a category, a title] ifFalse: [a title].
		second := b isCategorized ifTrue: [b category, b title] ifFalse: [b title].
		first < second]! !

!Object methodsFor: '*gt-inspector' stamp: 'TudorGirba 3/17/2014 16:29'!
gtInspectorExampleSourceIn: composite inContext: aGTInspector
	<gtInspectorPresentationOrder: 900>
	
	| previousRawSelection |
	(aGTInspector panes size > 1) 
		ifFalse: [ ^ self ].	
	previousRawSelection := (aGTInspector panes reversed second port: #rawSelection) value.
	(previousRawSelection isKindOf: GTExample)
			ifTrue: [ 
				(previousRawSelection methodReference gtInspectorSourceIn: composite) 
					title: 'E.g. source' ]! !

!Object methodsFor: '*gt-inspector' stamp: 'TudorGirba 3/17/2014 08:33'!
gtInspectorMetaIn: composite
	"This presentation offers the methods of the class of the current object"
	<gtInspectorPresentationOrder: 1000>
	^ composite custom: (GTSimpleMethodsBrowser new startOn: self) ! !

!Object methodsFor: '*gt-inspector' stamp: 'TudorGirba 3/13/2014 13:28'!
gtInspectorPresentationsIn: composite inContext: aGTInspector
	"This is a utility method that collects all presentations for the current object.
	By default, it simply looks for the #gtInspectorPresentationOrder: pragma.
	Subclasses might want to override it for more special behavior."
	| pragmas |
	pragmas := Pragma 
				allNamed: #gtInspectorPresentationOrder: 
				from: self class 
				to: Object
				sortedUsing: [: x :y | (x argumentAt: 1) < (y argumentAt: 1)].
	pragmas do: [ :eachPragma | 
		(eachPragma selector findTokens: $:) size = 1 
			ifTrue: [ self perform: eachPragma selector with: composite ]. 
		(eachPragma selector findTokens: $:) size = 2
			ifTrue: [ self perform: eachPragma selector with: composite with: aGTInspector ] ]! !

!Object methodsFor: '*gt-inspector' stamp: 'TudorGirba 3/17/2014 08:33'!
gtInspectorStateIn: composite
	"This is the most basic presentation showing the state of the object"
	<gtInspectorPresentationOrder: 10>
	^ composite custom: (GTObjectVariablesBrowser new startOn: self)! !

!Object methodsFor: '*gt-inspector' stamp: 'TudorGirba 1/28/2014 22:54'!
gtInspectorVariableValuePairs
	"This is a helper method that returns a collection of 
		variable_name -> value
	for the current object.
	Subclasses can override it to specialize what appears in the variables presentation"
	| bindings |
	bindings := self class allInstVarNames 
					collect: [ :iv | iv -> (self instVarNamed: iv) ].
	^ Set new
		addAll: bindings;
		yourself! !
GTInspector initialize!
